@@ -18,7 +18,7 @@ int main (int argc, char *argv[])
     [[GTKApplication alloc] initWithArgc:&argc argv:&argv];
 
     window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
-    [window borderWidth:10];
+    [window setBorderWidth:10];
 
     button = [GTKButton buttonWithLabel:@"Quit"];
     label = [GTKLabel labelWithStr:@"Hello World!"];
@@ -29,9 +29,8 @@ int main (int argc, char *argv[])
     [window add:box];
     [window show];
 
+    [button grabFocus];
     [button connectSignal:@"clicked" withTarget:window sel:@selector(destroy)];
-    [window connectSignal:@"destroy" withTarget:GTKApp
-	    sel:@selector(terminate:)];
 
     NSLog([[[box children] objectAtIndex:0] stringValue]);
 
