@@ -18,9 +18,11 @@
  * Boston, MA 02111-1307, USA.
  */
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <sys/types.h>
@@ -34,6 +36,8 @@ extern char *getcwd ();
 #include <Foundation/NSDictionary.h>
 #include "Manager.h"
 
+#define KEY(listitem)	[[listitem child] stringValue]
+
 @implementation Manager
 
 - init
@@ -49,8 +53,10 @@ extern char *getcwd ();
     GTKVSeparator *vsep = [GTKVSeparator vSeparator];
     GTKHBox *hbox2 = [GTKHBox hBoxWithHomogeneous:NO spacing:0];
     GTKMenu *popup = [GTKMenu menu];
-    GTKMenuFactory *menu_factory =
-	[GTKMenuFactory menuFactoryWithType:GTK_MENU_FACTORY_MENU_BAR];
+    GTKItemFactory *item_factory =
+	[GTKItemFactory itemFactoryWithContainerType:GTK_TYPE_MENU_BAR
+			path:@"<menu>" accelGroup:nil];
+    GTKWidget *menubar;
     GTKScrolledWindow *scroll1 =
 	[GTKScrolledWindow scrolledWindowWithHadjustment:nil vadjustment:nil];
     GTKScrolledWindow *scroll2 =
@@ -58,29 +64,30 @@ extern char *getcwd ();
     GTKAction *select =
 	[GTKAction actionWithTarget:self selector:@selector(select:)];
     GTKTooltips *tips = [GTKTooltips tooltips];
-
-    static GdkColor yellow, black;
-    static GtkMenuEntry menus[] =
-    {
-	{ "Project/Info Panel...",  "<control>I", 0, @selector(info:) },
-	{ "Project/Preferences..." },
-	{ "Project/<separator>" },
-	{ "File/Open",		    "<control>O", 0, @selector(open:) },
-	{ "File/<separator>" },
-	{ "File/Add...",	    "<control>A", 0, @selector(add:) },
-	{ "File/Remove",	    "<control>R", 0, @selector(remove:) },
-	{ }
-    };
-    static GtkMenuEntry menus2[] =
-    {
-	{ "Project/Quit",	    "<control>Q", 0, @selector(destroy) },
-	{ }
-    };
     int index;
     char buffer[1024];
 
+    static GTKItemFactoryEntry menus[] =
+    {
+	// target = self
+	{{ "/_Project", NULL, 0, 0, "<Branch>" }},
+	{{ "/Project/_Info Panel...",	"<control>I" },	@selector(info:) },
+	{{ "/Project/_Preferences..." }},
+	{{ "/Project/sep1", NULL, 0, 0, "<Separator>" }},
+
+	{{ "/_File", NULL, 0, 0, "<Branch>" }},
+	{{ "/File/_Add...",		"<control>A" },	@selector(add:) },
+	{{ "/File/sep2", NULL, 0, 0, "<Separator>" }},
+	{{ "/File/_Open",		"<control>O" },	@selector(open:) },
+	{{ "/File/_Remove",		"<control>R" },	@selector(remove:) },
+
+	// target = window
+	{{ "/Project/_Quit",		"<control>Q" },	@selector(destroy) },
+    };
+
     [super init];
 #ifdef HAVE_GETCWD
+    // this should be replaced with [filemanager currentDirectoryPath]
     if (getcwd(buffer, sizeof buffer))
 	path = [[NSString alloc] initWithCString:buffer];
     else
@@ -100,9 +107,9 @@ extern char *getcwd ();
 		      [GTKListItem listItemWithLabel:@"Libraries"], nil];
     [list1 appendItems:groups];
 
-    [menu_factory addEntries:menus forTarget:self];
-    [menu_factory addEntries:menus2 forTarget:window];
-    [GTK_GET_OBJECT(menus[1].widget) setSensitive:NO];
+    [item_factory createItems:&menus[0] count:9 forTarget:self];
+    [item_factory createItems:&menus[9] count:1 forTarget:window];
+    [[item_factory getWidget:@"/Project/Preferences..."] setSensitive:NO];
 
     target = [GTKOptionMenu optionMenu];
     [popup append:[GTKMenuItem menuItemWithLabel:@"(default)"]];
@@ -112,20 +119,14 @@ extern char *getcwd ();
 
     [scroll1 setPolicy:GTK_POLICY_AUTOMATIC vscrollbarPolicy:GTK_POLICY_ALWAYS];
     [scroll2 setPolicy:GTK_POLICY_AUTOMATIC vscrollbarPolicy:GTK_POLICY_ALWAYS];
-    [scroll1 add:list1];
-    [scroll2 add:list2];
+    [scroll1 addWithViewport:list1];
+    [scroll2 addWithViewport:list2];
 
     [launch connectSignal:@"clicked" withTarget:self sel:@selector(launch:)];
     [debug  connectSignal:@"clicked" withTarget:self sel:@selector(debug:)];
     [attrib connectSignal:@"clicked" withTarget:self sel:@selector(attrib:)];
     [build  connectSignal:@"clicked" withTarget:self sel:@selector(build:)];
 
-    gdk_color_parse("light yellow", &yellow);
-    gdk_color_alloc([window getColormap], &yellow);
-    gdk_color_parse("black", &black);
-    gdk_color_alloc([window getColormap], &black);
-
-    [tips setColors:&yellow foreground:&black];
     [tips setTip:launch tipText:@"Start the Program" tipPrivate:nil];
     [tips setTip:debug tipText:@"Start the Debugger" tipPrivate:nil];
     [tips setTip:attrib tipText:@"Modify Project Options" tipPrivate:nil];
@@ -140,13 +141,15 @@ extern char *getcwd ();
     [hbox1 packStart:target expand:NO fill:YES padding:4];
     [hbox2 packStart:scroll1 expand:YES fill:YES padding:4];
     [hbox2 packStart:scroll2 expand:YES fill:YES padding:4];
-    [vbox packStart:[menu_factory widget] expand:NO fill:YES padding:0];
+
+    menubar = [item_factory getWidget:@"<menu>"];
+    [vbox packStart:menubar expand:NO fill:YES padding:0];
     [vbox packStart:hbox1 expand:NO fill:NO padding:8];
     [vbox packStart:hsep expand:NO fill:YES padding:0];
     [vbox packStart:hbox2 expand:YES fill:YES padding:8];
 
     [window add:vbox];
-    [window addAcceleratorTable:[menu_factory acceleratorTable]];
+    [window addAccelGroup:[item_factory accelGroup]];
     [window connectSignal:@"destroy" withTarget:GTKApp
 	    sel:@selector(terminate:)];
     [window setUsize:320 height:220];
@@ -159,7 +162,7 @@ extern char *getcwd ();
     {
 	GTKListItem *item = [groups objectAtIndex:--index];
 
-	[files setObject:[NSMutableArray array] forKey:(id) item];
+	[files setObject:[NSMutableArray array] forKey:KEY(item)];
 	[item connectSignal:@"select" withAction:select];
     }
 
@@ -187,7 +190,7 @@ extern char *getcwd ();
 - (void) select:sender
 {
     [list2 removeItems:[list2 children]];
-    [list2 appendItems:[files objectForKey:sender]];
+    [list2 appendItems:[files objectForKey:KEY(sender)]];
     if ([GTKApp getCurrentEvent]->type == GDK_2BUTTON_PRESS)
 	[self addFile:sender];
 }
@@ -203,11 +206,13 @@ extern char *getcwd ();
     NSArray *sel = [list1 selection];
 
     if ([sel count]) [self addFile:[sel objectAtIndex:0]];
+    else GTKRunMessageBox(@"Warning", @"You must select a group first",
+			  @"Continue", nil, nil);
 }
 
 - (void) addFile:(GTKListItem *) group
 {
-    NSMutableArray *items = [files objectForKey:(id) group];
+    NSMutableArray *items = [files objectForKey:KEY(group)];
     GTKFileSelection *filesel =
 	[[GTKFileSelection alloc] initWithTitle:@"Add File..."];
 
@@ -252,7 +257,7 @@ extern char *getcwd ();
     GTKListItem *item;
 
     if ([sel1 count] == 0 || [sel2 count] == 0) return;
-    items = [files objectForKey:[sel1 objectAtIndex:0]];
+    items = [files objectForKey:KEY([sel1 objectAtIndex:0])];
     item = [sel2 objectAtIndex:0];
 
     [list2 remove:item];
@@ -266,7 +271,7 @@ extern char *getcwd ();
     NSString *command;
 
     if ([sel2 count] == 0) return;
-    command = [NSString stringWithFormat:@"%@ '%@' &", TEXT_EDITOR,
+    command = [NSString stringWithFormat:@"%@ '%@' &", EDIT_CMD,
 			[[[sel2 objectAtIndex:0] child] stringValue]];
     system([command cString]);
 }
@@ -290,13 +295,14 @@ extern char *getcwd ();
 - (void) attrib:sender
 {
     NSString *command = [NSString stringWithFormat:@"%@ '%@' &",
-				  TEXT_EDITOR, @"Makefile"];
+				  EDIT_CMD, @"Makefile"];
 
     system([command cString]);
 }
 
 - (void) build:sender
 {
+    static GdkFont *font;
     GTKWindow *output = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
     GTKHBox *box = [GTKHBox hBoxWithHomogeneous:NO spacing:4];
     GTKVScrollbar *bar = [GTKVScrollbar vScrollbarWithAdjustment:nil];
@@ -305,11 +311,13 @@ extern char *getcwd ();
     FILE *pipe;
     char buffer[1024];
 
+    if (!font) font = gdk_font_load(OUTPUT_FONT);
+
     [text setEditable:NO];
     [box packStart:text expand:YES fill:YES padding:0];
     [box packStart:bar expand:NO fill:YES padding:0];
     [output setTitle:@"Build Output Window"];
-    [output borderWidth:2];
+    [output setBorderWidth:2];
     [output add:box];
     [output setUsize:480 height:240];
     [output show];
@@ -322,12 +330,12 @@ extern char *getcwd ();
     if ((pipe = popen([command cString], "r")))
     {
 	while (fgets(buffer, sizeof buffer, pipe))
-	    [text insert:NULL fore:NULL back:NULL
+	    [text insert:font fore:NULL back:NULL
 		  chars:[NSString stringWithCString:buffer] length:-1];
 	pclose(pipe);
     }
 
-    [text insert:NULL fore:NULL back:NULL chars:@"--- finished ---" length:-1];
+    [text insert:font fore:NULL back:NULL chars:@"--- finished ---" length:-1];
 }
 
 - (void) loadFromFile:(NSString *) _filename
@@ -351,7 +359,7 @@ extern char *getcwd ();
     keys = [groups objectEnumerator];
     while ((item = [keys nextObject]))
     {
-	NSMutableArray *array = [files objectForKey:(id) item];
+	NSMutableArray *array = [files objectForKey:KEY(item)];
 	char buffer[4096];
 	char *ptr, *string;
 
@@ -399,8 +407,8 @@ extern char *getcwd ();
     keys = [groups objectEnumerator];
     while ((item = [keys nextObject]))
     {
-	fprintf(file, "%s =", [[[item child] stringValue] cString]);
-	values = [[files objectForKey:(id) item] objectEnumerator];
+	fprintf(file, "%s =", [KEY(item) cString]);
+	values = [[files objectForKey:KEY(item)] objectEnumerator];
 	while ((item = [values nextObject]))
 	{
 	    string = [[item child] stringValue];
