@@ -1,5 +1,6 @@
 #include <GToolKit/GToolKit.h>
 #include <Foundation/NSAutoreleasePool.h>
+#include <Foundation/NSArray.h>
 
 @interface Foo : NSObject
 {
@@ -18,22 +19,27 @@
     GTKWindow *window;
     GTKVBox *box;
     GTKAdjustment *adjust;
+    GTKCombo *combo;
     GTKButton *button;
 
     [super init];
 
+    window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
+    button = [GTKButton buttonWithLabel:@"Colors..."];
     adjust = [GTKAdjustment adjustmentWithValue:50 lower:0 upper:100
-	      stepIncrement:1 pageIncrement:10 pageSize:0];
+			    stepIncrement:1 pageIncrement:10 pageSize:0];
     box = [GTKVBox vBoxWithHomogeneous:YES spacing:10];
     scale = [GTKHScale hScaleWithAdjustment:adjust];
-    entry = [GTKEntry entry];
-    button = [GTKButton buttonWithLabel:@"Colors..."];
-    window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
+    combo = [GTKCombo combo];
+    entry = [combo entry];
+
+    [combo setPopdownStrings:
+	   [NSArray arrayWithObjects:@"0", @"25", @"50", @"75", @"100", nil]];
 
     [box packStartDefaults:scale];
-    [box packStartDefaults:entry];
+    [box packStartDefaults:combo];
     [box packStartDefaults:button];
-    [window borderWidth:10];
+    [window setBorderWidth:10];
     [window add:box];
     [window show];
 
