@@ -20,14 +20,18 @@
 #ifndef GTOOLKIT_GTK_H
 #define GTOOLKIT_GTK_H
 
-#include <glib.h>
+#define GTK_DISABLE_COMPAT_H
+
 #include <Foundation/NSObject.h>
 #include <Foundation/NSString.h>
+#include <gtk/gtkenums.h>
+#include <glib.h>
 
 @class NSArray, NSMutableArray, NSNotification;
 @class GTKApplication, GTKAction;
 
-@class GTKAcceleratorTable;
+@class GTKAccelGroup;
+@class GTKAccelLabel;
 @class GTKAdjustment;
 @class GTKAlignment;
 @class GTKArrow;
@@ -37,6 +41,8 @@
 @class GTKButton;
 @class GTKButtonBox;
 @class GTKCList;
+@class GTKCTree;
+@class GTKCalendar;
 @class GTKCheckButton;
 @class GTKCheckMenuItem;
 @class GTKColorSelection;
@@ -52,6 +58,8 @@
 @class GTKEventBox;
 @class GTKFileSelection;
 @class GTKFixed;
+@class GTKFontSelection;
+@class GTKFontSelectionDialog;
 @class GTKFrame;
 @class GTKGammaCurve;
 @class GTKHBox;
@@ -64,8 +72,11 @@
 @class GTKHandleBox;
 @class GTKImage;
 @class GTKInputDialog;
+@class GTKInvisible;
 @class GTKItem;
+@class GTKItemFactory;
 @class GTKLabel;
+@class GTKLayout;
 @class GTKList;
 @class GTKListItem;
 @class GTKMenu;
@@ -77,9 +88,12 @@
 @class GTKNotebook;
 @class GTKObject;
 @class GTKOptionMenu;
+@class GTKPacker;
 @class GTKPaned;
 @class GTKPixmap;
+@class GTKPlug;
 @class GTKPreview;
+@class GTKProgress;
 @class GTKProgressBar;
 @class GTKRadioButton;
 @class GTKRadioMenuItem;
@@ -89,10 +103,12 @@
 @class GTKScrollbar;
 @class GTKScrolledWindow;
 @class GTKSeparator;
+@class GTKSocket;
 @class GTKSpinButton;
 @class GTKStatusbar;
 @class GTKStyle;
 @class GTKTable;
+@class GTKTearoffMenuItem;
 @class GTKText;
 @class GTKTipsQuery;
 @class GTKToggleButton;
@@ -111,19 +127,20 @@
 @class GTKWidget;
 @class GTKWindow;
 
-#define String_to_NSString(s)	[NSString stringWithCString:s]
-#define NSString_to_String(s)	(char *) [(NSString *)(s) cString]
+typedef struct _GTKItemFactoryEntry GTKItemFactoryEntry;
 
-extern NSArray *GList_to_NSArray (GList *list);
+#define String_to_NSString(s)	[NSString stringWithCString:s]
+#define NSString_to_String(s)	(char *)[(NSString *)(s) cString]
+
+extern NSArray *GList_to_NSArray (GList *glist);
 extern GList *NSArray_to_GList (NSArray *array);
+
+extern NSArray *StrVec_to_NSArray (const char **strvec);
 extern const char **NSArray_to_StrVec (NSArray *array);
 
-#define GSList_to_NSArray(l)	(NSArray *) (l)
-#define NSArray_to_GSList(a)	(GSList *) (a)
-
 /* Find Objective-C object for given gtk-object pointer */
-#define GTK_GET_OBJECT(gtk)	Gtk_to_Object(gtk,0)
 extern id Gtk_to_Object (gpointer gtk, const char *class);
+#define GTK_GET_OBJECT(gtk)	Gtk_to_Object(gtk,0)
 
 /*
  * GTK is the base class of all classes in the Objective-C interface to
@@ -138,7 +155,7 @@ extern id Gtk_to_Object (gpointer gtk, const char *class);
 @interface GTK : NSObject
 {
     gpointer gtk;		// internal gtk-object pointer
-    gint32 tag;			// assigned tag
+    gint32 tag;			// assigned tag value
     NSMutableArray *actions;	// array of connected actions
 }
 
@@ -151,8 +168,8 @@ extern id Gtk_to_Object (gpointer gtk, const char *class);
 - (void) setTag:(gint32) tag;
 - (gint32) tag;
 
-/* signal handling */
-+ (void) addUserSignal:(NSString *) signal;
+/* signal handling for GTKObject */
++ (void) addUserSignal:(NSString *) signal flags:(GtkSignalRunType) flags;
 - (void) emit:sender signal:(NSString *) signal;
 - (void) stop:sender signal:(NSString *) signal;
 - (void) connectSignal:(NSString *) signal withAction:(GTKAction *) action;
