@@ -19,35 +19,17 @@
  */
 #include <ctype.h>
 #include <stdio.h>
-#include <Foundation/NSArray.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSObjCRuntime.h>
+#include <Foundation/NSArray.h>
 #include <Foundation/NSMapTable.h>
 #include <Foundation/NSLock.h>
 #include <Foundation/NSThread.h>
-
-#ifndef DEBUG
-#define GTK_NO_CHECK_CASTS
-#endif
 #include <GToolKit/GToolKit.h>
 
 /*
- * Some general type conversion functions for NSArray.
+ * Objects of this class can autorelease standard C pointers.
  */
-static void add_to_array (gpointer data, NSMutableArray *array)
-{
-    [array addObject:Gtk_to_Object(data, NULL)];
-}
-
-NSArray *GList_to_NSArray (GList *list)
-{
-    NSMutableArray *array;
-
-    array = [NSMutableArray arrayWithCapacity:g_list_length(list)];
-    g_list_foreach(list, (GFunc) add_to_array, array);
-    return array;
-}
-
 @interface __GAutorelease : NSObject
 {
     gpointer object;
@@ -77,14 +59,45 @@ NSArray *GList_to_NSArray (GList *list)
 }
 @end
 
+/*
+ * Some general type conversion functions for NSArray.
+ */
+static void add_to_array (gpointer data, NSMutableArray *array)
+{
+    // XXX: this will only work if data is some kind of GtkObject...
+    [array addObject:Gtk_to_Object(data, NULL)];
+}
+
+NSArray *GList_to_NSArray (GList *glist)
+{
+    NSMutableArray *array;
+
+    array = [NSMutableArray arrayWithCapacity:g_list_length(glist)];
+    g_list_foreach(glist, (GFunc) add_to_array, array);
+
+    return array;
+}
+
 GList *NSArray_to_GList (NSArray *array)
 {
     GList *list = NULL;
     int index, count = [array count];
 
     for (index = 0; index < count; ++index)
-	list = g_list_append(list, [[array objectAtIndex:index] gtk]);
+    {
+	id object = [array objectAtIndex:index];
+	const void *data;
 
+	data = [object isKindOfClass:[GTK class]] ? [object gtk]
+	     : [object isKindOfClass:[NSString class]] ? [object cString]
+//	     : [object isKindOfClass:[NSData class]] ? [object bytes]
+	     : [object isKindOfClass:[NSArray class]] ? NSArray_to_GList(object)
+	     : (gpointer) [[object description] cString];
+
+	list = g_list_append(list, (gpointer) data);
+    }
+
+    // XXX: check if this is still necessary
 #ifdef GLIST_AUTORELEASE
     return [__GAutorelease release:list with:(void (*)(gpointer)) g_list_free];
 #else
@@ -92,11 +105,28 @@ GList *NSArray_to_GList (NSArray *array)
 #endif /* GLIST_AUTORELEASE */
 }
 
-extern const char **NSArray_to_StrVec (NSArray *array)
+NSArray *StrVec_to_NSArray (const char **strvec)
+{
+    NSMutableArray *array = nil;
+
+    if (strvec)
+    {
+	array = [NSMutableArray arrayWithCapacity:4];
+	while (*strvec)
+	    [array addObject:[NSString stringWithCString:*strvec++]];
+    }
+
+    return array;
+}
+
+const char **NSArray_to_StrVec (NSArray *array)
 {
     int index, count = [array count];
-    const char **vec = g_new(const char *, count + 1);
+    const char **vec;
 
+    if (array == nil) return NULL;
+
+    vec = g_new(const char *, count + 1);
     vec[count] = NULL;
     for (index = 0; index < count; ++index)
 	vec[index] = [[array objectAtIndex:index] cString];
@@ -104,6 +134,7 @@ extern const char **NSArray_to_StrVec (NSArray *array)
     return [__GAutorelease release:vec with:g_free];
 }
 
+extern BOOL gtoolkit_debug;		// enable limited debug output
 static NSLock *global_lock;		// lock for all global variables
 static NSMapTable *object_table;	// map gpointer to object
 
@@ -115,31 +146,30 @@ static NSMapTable *object_table;	// map gpointer to object
  */
 id Gtk_to_Object (gpointer gtk, const char *class)
 {
-    id result = nil;
+    id result;
 
-    if (gtk)
+    if (gtk == NULL) return nil;
+
+    if (global_lock) [global_lock lock];
+    result = NSMapGet(object_table, gtk);
+    if (global_lock) [global_lock unlock];
+
+    if (result == nil)
     {
-	[global_lock lock];
-	result = NSMapGet(object_table, gtk);
-	[global_lock unlock];
+	NSString *name = [NSString alloc];
 
-	if (result == nil)
+	if (class) name = [name initWithCString:class];
+	else
 	{
-	    NSString *name = [NSString alloc];
+	    char *str = g_strdup(gtk_type_name(GTK_OBJECT_TYPE(gtk)));
 
-	    if (class) name = [name initWithCString:class];
-	    else
-	    {
-		char *str = g_strdup(gtk_type_name(GTK_OBJECT_TYPE(gtk)));
-
-		if (str[0]) str[1] = toupper(str[1]);	// GtkFoo -> GTKFoo
-		if (str[1]) str[2] = toupper(str[2]);
-		name = [name initWithCString:str];
-		g_free(str);
-	    }
-	    result = [[NSClassFromString(name) alloc] initWithGtk:gtk];
-	    [name release];
+	    if (str[0]) str[1] = toupper(str[1]);	// GtkFoo -> GTKFoo
+	    if (str[1]) str[2] = toupper(str[2]);
+	    name = [name initWithCString:str];
+	    g_free(str);
 	}
+	result = [[NSClassFromString(name) alloc] initWithGtk:gtk];
+	[name release];
     }
 
     return result;
@@ -150,7 +180,7 @@ id Gtk_to_Object (gpointer gtk, const char *class)
  * This method is intended only for use by the @GTKApplication class
  * and should not be called from user code.
  */
-+ (void) taskNowMultiThreaded:(NSNotification *) event;
++ (void) taskNowMultiThreaded:(NSNotification *) event
 {
     if (global_lock == nil) global_lock = [NSLock new];
 }
@@ -175,7 +205,7 @@ id Gtk_to_Object (gpointer gtk, const char *class)
  - init
 {
     g_error("Objects of type `%s' cannot be created with +new",
-	    [[[self class] description] cString]);
+	    [[isa description] cString]);
     return nil;
 }
 
@@ -194,19 +224,16 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 					NSNonOwnedPointerMapValueCallBacks, 64);
     NSMapInsert(object_table, gtk = _gtk, self);
     [global_lock unlock];
-#ifdef DEBUG
-    fprintf(stderr, "[%s init] at %p: gtk = %p\n",
-	    [[[self class] description] cString], self, gtk);
-#endif
+
+    if (gtoolkit_debug) fprintf(stderr, "[%s init] at %p: gtk = %p\n",
+				[[isa description] cString], self, gtk);
     return self;
 }
 
  - (void) dealloc
 {
-#ifdef DEBUG
-    fprintf(stderr, "[%s dealloc] at %p: gtk = %p\n",
-	    [[[self class] description] cString], self, gtk);
-#endif
+    if (gtoolkit_debug) fprintf(stderr, "[%s dealloc] at %p: gtk = %p\n",
+				[[isa description] cString], self, gtk);
     [global_lock lock];
     NSMapRemove(object_table, gtk);
     [global_lock unlock];
@@ -226,8 +253,9 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 
 /*
  * Return the internal gtk-object pointer. This may be generally useful to
- * access components of the underlying object that do not have access
- * methods. Example:
+ * access components of the underlying gtk object that do not have access
+ * methods. The macro call |GTK_GET_OBJECT(|/gtk_pointer/|)| can perform the
+ * reverse operation. Example:
  * <pre>
  * GTKStyle *style = [button getStyle];
  * GtkStyle *gtk_style = [style gtk];
@@ -259,13 +287,15 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 
 /*
  * Add a user defined /signal/ (without parameters) to the target class.
- * The signal can be emitted by sending @-emit:signal: to an object of
+ * /flags/ may be either |0| or any valid combination of the signal flags
+ * defined in the header file gtk/gtkenums.h.
+ * The new signal can be emitted by sending @-emit:signal: to an object of
  * this class.
  */
-+ (void) addUserSignal:(NSString *) signal
++ (void) addUserSignal:(NSString *) signal flags:(GtkSignalRunType) flags
 {
-    gtk_object_class_add_user_signal(gtk_type_class([self getType]),
-				     NSString_to_String(signal),
+    gtk_object_class_user_signal_new(gtk_type_class([self getType]),
+				     NSString_to_String(signal), flags,
 				     gtk_signal_default_marshaller,
 				     GTK_TYPE_NONE, 0);
 }
@@ -334,8 +364,8 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  */
 - (void) connectSignal:(NSString *) signal withAction:(GTKAction *) action
 {
-    gtk_signal_connect_interp(gtk, NSString_to_String(signal),
-			(GtkCallbackMarshal) forward_signal, action, NULL, NO);
+    gtk_signal_connect_full(gtk, NSString_to_String(signal), NULL,
+		    (GtkCallbackMarshal) forward_signal, action, NULL, NO, NO);
     if (actions == nil) actions = [NSMutableArray new];
     [actions addObject:action];
 }
@@ -380,8 +410,8 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  */
 - (void) connectSignal:(NSString *) signal withActionAfter:(GTKAction *) action
 {
-    gtk_signal_connect_interp(gtk, NSString_to_String(signal),
-			(GtkCallbackMarshal) forward_signal, action, NULL, YES);
+    gtk_signal_connect_full(gtk, NSString_to_String(signal), NULL,
+		    (GtkCallbackMarshal) forward_signal, action, NULL, NO, YES);
     if (actions == nil) actions = [NSMutableArray new];
     [actions addObject:action];
 }
