@@ -20,6 +20,10 @@
 #include <GToolKit/GTKAction.h>
 #include <GToolKit/GTK.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 @implementation GTKAction
 
 /*
@@ -135,18 +139,18 @@
 {
 #define MAXARGS	3
     int (*func)(id, SEL, ...) = (int (*)(id, SEL, ...)) method;
-    int is_int[MAXARGS];
-    int num[MAXARGS];
-    void *ptr[MAXARGS];
+    int is_int[MAXARGS];	// is this type "compatible" to int?
+    int num[MAXARGS];		// integer value of the argument
+    void *ptr[MAXARGS];		// pointer value of the argument
     int index;
     int result;
 
     // this code currently does not use the NSInvocation class because:
-    //	 - it is implemented only on a few systems (yet)
-    //	 - it is relatively slow
+    //	 - NSInvocation is implemented only on a few systems (yet)
+    //	 - NSInvocation is relatively slow
 
-    // allow up to MAXARGS arguments of type INT, LONG or POINTER
-    for (index = 0; index < MAXARGS; ++index)
+    // allow up to MAXARGS arguments of type INT, LONG, FLOAT or POINTER
+    for (index = 0; index != MAXARGS; ++index)
     {
 	if (index < count)
 	{
@@ -156,21 +160,33 @@
 	    if (type == GTK_TYPE_LONG || type == GTK_TYPE_ULONG)
 		g_warning("cannot pass GTK_TYPE_LONG on this machine");
 #endif
-	    is_int[index] = type == GTK_TYPE_INT  || type == GTK_TYPE_UINT  ||
-#if SIZEOF_LONG == SIZEOF_INT
-			    type == GTK_TYPE_LONG || type == GTK_TYPE_ULONG ||
+#if SIZEOF_FLOAT != SIZEOF_INT && SIZEOF_FLOAT != SIZEOF_VOID_P
+	    if (type == GTK_TYPE_FLOAT)
+		g_warning("cannot pass GTK_TYPE_FLOAT on this machine");
 #endif
-			    type == GTK_TYPE_ENUM || type == GTK_TYPE_FLAGS;
+	    is_int[index] = type == GTK_TYPE_INT   || type == GTK_TYPE_UINT  ||
+#if SIZEOF_INT == SIZEOF_LONG
+			    type == GTK_TYPE_LONG  || type == GTK_TYPE_ULONG ||
+#endif
+			    type == GTK_TYPE_BOOL  || type == GTK_TYPE_ENUM  ||
+#if SIZEOF_INT == SIZEOF_FLOAT
+			    type == GTK_TYPE_FLOAT ||
+#endif
+			    type == GTK_TYPE_FLAGS;
 	    num[index] = GTK_VALUE_INT(args[index]);
 	    ptr[index] = gtk_type_is_a(type, GTK_TYPE_OBJECT) ?
 			    Gtk_to_Object(GTK_VALUE_OBJECT(args[index]), 0) :
 			    GTK_VALUE_POINTER(args[index]);
 	}
-	else is_int[index] = 0, num[index] = 0, ptr[index] = data; 
+	else
+	{
+	    is_int[index] = 0, ptr[index] = data;
+	    break;
+	}
     }
 
     // method is always called with the maximum number of arguments,
-    // this is OK because Objective-C uses the C calling convention
+    // this is ok because Objective-C uses the C calling convention
     result =
     (is_int[0] ?
 	(is_int[1] ?
