@@ -19,6 +19,7 @@
  */
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSException.h>
@@ -27,15 +28,24 @@
 #include <Foundation/NSUtilities.h>
 #include <GToolKit/GToolKit.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#if !defined HAVE_NSTHREAD_NOTIFICATION && !defined LIB_FOUNDATION_LIBRARY
+#define NSWillBecomeMultiThreadedNotification	NSBecomingMultiThreaded
+#define NSThreadWillExitNotification		NSThreadExiting
+#endif
+
 static void clicked (GtkWidget *widget)
 {
-    [GTKApp stopModalWithCode:[Gtk_to_Object(widget, 0) tag]];
+    [GTKApp stopModalWithCode:[GTK_GET_OBJECT(widget) tag]];
 }
 
 static gint delete (GtkWidget *widget, GdkEvent *event)
 {
     clicked(widget);
-    return NO;
+    return YES;
 }
 
 static GTKButton *insert (GTKBox *box, NSString *label, int tag)
@@ -71,7 +81,7 @@ int GTKRunMessageBox (NSString *title, NSString *format, NSString *button1,
     if (button1) focus = insert(hbox, button1, tag);
     if (button2) insert(hbox, button2, ++tag);
     if (button3) insert(hbox, button3, ++tag);
-    if (focus) [window setDefault:focus], [window setFocus:focus];
+    if (focus) [focus grabDefault], [focus grabFocus];
     if (title) [window setTitle:title];
 
     va_start(ap, button3);
@@ -91,6 +101,7 @@ int GTKRunMessageBox (NSString *title, NSString *format, NSString *button1,
     return result;
 }
 
+BOOL gtoolkit_debug;			// enable limited debug output
 static NSLock *gtkapp_lock;		// lock for GTKApp instance variables
 
 @implementation GTKApplication
@@ -106,7 +117,7 @@ NSString *GTKApplicationWillFinishLaunchingNotification =
 NSString *GTKApplicationWillTerminateNotification =
 	@"GTKApplicationWillTerminate";
 
-GTKApplication *GTKApp;		// the application object
+GTKApplication *GTKApp;			// the application object
 
 /*
  * Return the shared application object (as stored in the variable GTKApp).
@@ -121,7 +132,7 @@ GTKApplication *GTKApp;		// the application object
     return GTKApp;
 }
 
- + (void) taskNowMultiThreaded:(NSNotification *) event;
+ + (void) taskNowMultiThreaded:(NSNotification *) event
 {
 #ifdef DEBUG
     fprintf(stderr, "[GTKApplication taskNowMultiThreaded]\n");
@@ -185,9 +196,9 @@ GTKApplication *GTKApp;		// the application object
     count = [windows count];
     [gtkapp_lock unlock];
 
-    if (count == 0 && [delegate respondsToSelector:
-	@selector(applicationShouldTerminateAfterLastWindowClosed:)] &&
-	[delegate applicationShouldTerminateAfterLastWindowClosed:self])
+    if (count == 0 && (![delegate respondsToSelector:
+	@selector(applicationShouldTerminateAfterLastWindowClosed:)] ||
+	[delegate applicationShouldTerminateAfterLastWindowClosed:self]))
 	[self terminate:self];
 }
 
@@ -208,15 +219,20 @@ GTKApplication *GTKApp;		// the application object
  * variable GTKApp. Note that you cannot create more than one application
  * object in your program. This method will modify the program's command
  * line arguments as given by /argc/ and /argv/ and remove any options
- * related to GTK (such as |--display| or |--class|) from the argument list.
+ * related to GTK (such as |--display| or |--class|) from the argument
+ * list.<p>
+ * If the environment variable |GTOOLKIT_DEBUG| has been set, the
+ * GTKApplication class will activate special debug code to trace
+ * object allocation and deallocation calls.
  */
 - initWithArgc:(int *) argc argv:(char ***) argv
 {
-#ifdef DEBUG
-    fprintf(stderr, "[GTKApp initWithArgc:argv:]\n");
-#endif
     if (GTKApp == nil)
     {
+#ifdef DEBUG
+	fprintf(stderr, "[GTKApp initWithArgc:argv:]\n");
+	gtoolkit_debug = YES;
+#endif
 	GTKApp = [super init];
 	center = [NSNotificationCenter defaultCenter];
 	windows = [NSMutableArray new];
@@ -224,6 +240,8 @@ GTKApplication *GTKApp;		// the application object
 	gtk_set_locale();
 	gtk_init(argc, argv);
 
+	if (getenv("GTOOLKIT_DEBUG")) gtoolkit_debug = YES;
+
 	if ([NSThread isMultiThreaded])
 	    [GTKApplication taskNowMultiThreaded:nil];
 	else
@@ -320,6 +338,7 @@ GTKApplication *GTKApp;		// the application object
 {
     [self finishLaunching];
     [self main];
+    if ([NSThread isMultiThreaded]) [self exit:0];	// terminate safely
 }
 
 /*
@@ -376,7 +395,7 @@ GTKApplication *GTKApp;		// the application object
 }
 
 /*
- * Stop the application's main event loop (if it is running). 
+ * Stop the application's main event loop (if it is running).
  * This will cause the @-run method to return (similar to @-terminate:),
  * but will not send any notifications. If a modal even loop is running,
  * it will be stopped instead of the main event loop.
@@ -402,12 +421,16 @@ GTKApplication *GTKApp;		// the application object
  */
 - (void) terminate:sender
 {
+    static BOOL terminated;
+
+    if (terminated) return;
 #ifdef DEBUG
     fprintf(stderr, "[GTKApp terminate] sender = %p\n", sender);
 #endif
     if ([delegate respondsToSelector:@selector(applicationShouldTerminate:)] &&
 	[delegate applicationShouldTerminate:self] == NO) return;
 
+    terminated = YES;
     [center postNotificationName:GTKApplicationWillTerminateNotification
 	    object:self];
 
